lkml.org 
[lkml]   [2021]   [Jan]   [28]   [last100]   RSS Feed
Views: [wrap][no wrap]   [headers]  [forward] 
 
Messages in this thread
Patch in this message
/
From
Subject[PATCH v3 13/25] mm: Add unlock_folio
Date
Convert unlock_page() to call unlock_folio().  By using a folio we avoid
a call to compound_head(). This shortens the function from 39 bytes to
25 and removes 4 instructions on x86-64. Those instructions are currently
pushed into each caller, but subsequent patches will convert many of the
callers to operate on folios.

Signed-off-by: Matthew Wilcox (Oracle) <willy@infradead.org>
---
include/linux/pagemap.h | 16 +++++++++++++++-
mm/filemap.c | 27 ++++++++++-----------------
2 files changed, 25 insertions(+), 18 deletions(-)

diff --git a/include/linux/pagemap.h b/include/linux/pagemap.h
index 86956e97cd5e..5fcab5e1787c 100644
--- a/include/linux/pagemap.h
+++ b/include/linux/pagemap.h
@@ -623,9 +623,23 @@ extern int __lock_page_killable(struct page *page);
extern int __lock_page_async(struct page *page, struct wait_page_queue *wait);
extern int __lock_page_or_retry(struct page *page, struct mm_struct *mm,
unsigned int flags);
-extern void unlock_page(struct page *page);
+void unlock_folio(struct folio *folio);
extern void unlock_page_fscache(struct page *page);

+/**
+ * unlock_page - Unlock a locked page.
+ * @page: The page.
+ *
+ * Unlocks the page and wakes up any thread sleeping on the page lock.
+ *
+ * Context: May be called from interrupt or process context. May not be
+ * called from NMI context.
+ */
+static inline void unlock_page(struct page *page)
+{
+ return unlock_folio(page_folio(page));
+}
+
/*
* Return true if the page was successfully locked
*/
diff --git a/mm/filemap.c b/mm/filemap.c
index 4417fd15d633..b639651d1573 100644
--- a/mm/filemap.c
+++ b/mm/filemap.c
@@ -1407,29 +1407,22 @@ static inline bool clear_bit_unlock_is_negative_byte(long nr, volatile void *mem
#endif

/**
* unlock_page - unlock a locked page
* @page: the page
*
- * Unlocks the page and wakes up sleepers in wait_on_page_locked().
- * Also wakes sleepers in wait_on_page_writeback() because the wakeup
- * mechanism between PageLocked pages and PageWriteback pages is shared.
- * But that's OK - sleepers in wait_on_page_writeback() just go back to sleep.
+ * Unlocks the folio and wakes up any thread sleeping on the page lock.
*
- * Note that this depends on PG_waiters being the sign bit in the byte
- * that contains PG_locked - thus the BUILD_BUG_ON(). That allows us to
- * clear the PG_locked bit and test PG_waiters at the same time fairly
- * portably (architectures that do LL/SC can test any bit, while x86 can
- * test the sign bit).
+ * Context: May be called from interrupt or process context. May not be
+ * called from NMI context.
*/
-void unlock_page(struct page *page)
+void unlock_folio(struct folio *folio)
{
BUILD_BUG_ON(PG_waiters != 7);
- page = compound_head(page);
- VM_BUG_ON_PAGE(!PageLocked(page), page);
- if (clear_bit_unlock_is_negative_byte(PG_locked, &page->flags))
- wake_up_page_bit(page, PG_locked);
+ VM_BUG_ON_FOLIO(!FolioLocked(folio), folio);
+ if (clear_bit_unlock_is_negative_byte(PG_locked, folio_flags(folio)))
+ wake_up_page_bit(&folio->page, PG_locked);
}
-EXPORT_SYMBOL(unlock_page);
+EXPORT_SYMBOL(unlock_folio);

/**
* unlock_page_fscache - Unlock a page pinned with PG_fscache
@@ -1407,29 +1407,22 @@ static inline bool clear_bit_unlock_is_negative_byte(long nr, volatile void *mem

/**
- * unlock_page - unlock a locked page
- * @page: the page
+ * unlock_folio - Unlock a locked folio.
+ * @folio: The folio.
*
--
2.29.2
\
 
 \ /
  Last update: 2021-01-28 08:08    [W:0.160 / U:0.196 seconds]
©2003-2020 Jasper Spaans|hosted at Digital Ocean and TransIP|Read the blog|Advertise on this site